console.log ("v8.0.1");

/* eslint-env browser, es6 */

'use strict';

/* Define blank userSubscriptionInfo object */
var userSubscriptionInfo = {};

var ownScript = document.currentScript;
var scriptUrl = ownScript.src;

/* Reusable cookie function. Input the cookie name, value, and number ofdays until it should expire */
function setCookie(name,value,days) {
    var expires = "";
    if (days) {
        var date = new Date();
        date.setTime(date.getTime() + (days*24*60*60*1000));
        expires = "; expires=" + date.toUTCString();
    }
    document.cookie = name + "=" + (value || "")  + expires + "; path=/; Secure;";
}

/* Function to delete a cookie based on it's name */
function deleteCookie(name) {
    var expires = "expires=Thu, 01-Jan-1970 00:00:01 GMT";
    document.cookie = name + "=" + (value || "")  + expires + "; path=/; Secure;";
}

/* Pass in a cookie name to get it's perameters returned to you */
function getCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for(var i=0; i<ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') c = c.substring(1);
        if (c.indexOf(name) != -1) return c.substring(name.length,c.length);
    }
    return "";
}

/* Capture URL parameters from installation code */
function getUrlVars() {
    var vars = {};
    var parts = scriptUrl.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m,key,value) {
        vars[key] = value;
    });
    return vars;
}

function getUrlParam(parameter, defaultvalue){
    var urlparameter = defaultvalue;
    if(scriptUrl.indexOf(parameter) > -1){
        urlparameter = getUrlVars()[parameter];
        }
    return urlparameter;
}

var scriptUrlParams = getUrlVars ();

/* If this script's URL params include a 'delay' value, then wait the time specified time
   before showing the subscribe dialog */
delaySubPrompt = scriptUrlParams ['delay'];
if (delaySubPrompt == parseInt(delaySubPrompt, 10)) {
	delaySubPrompt = delaySubPrompt * 1000;
    console.log ("delay: " + delaySubPrompt);
} else {
	delaySubPrompt = 0;
}

/* Get the push widget ID, current page's hostname and user's cookied push ID 
   and append to the userSubscriptionInfo object */
userSubscriptionInfo.widgetId = scriptUrlParams ['wid'];
userSubscriptionInfo.domainServed = window.location.hostname;
userSubscriptionInfo.pushId = getCookie('pushId');

const applicationServerPublicKey = 'BARBSr_w7zQ9m7TxejodXmT-1Rf9fhA8XPPefk831dewdALPyspDFJW8sZP2XGY7uyMsAkN3oSjZ2IW5rAWueHQ=';

/* Identify the subscription button in the DOM (need to identify a use case) */
const pushButton = document.getElementById('notificationBtn');

let isSubscribed = false;
let swRegistration = null;

/* Convert a base64 string to a Unit8Array
   https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding
   https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Uint8Array */
function urlB64ToUint8Array(base64String) {
  const padding = '='.repeat((4 - base64String.length % 4) % 4);
  const base64 = (base64String + padding)
    .replace(/\-/g, '+')
    .replace(/_/g, '/');

  const rawData = window.atob(base64);
  const outputArray = new Uint8Array(rawData.length);

  for (let i = 0; i < rawData.length; ++i) {
    outputArray[i] = rawData.charCodeAt(i);
  }
  return outputArray;
}

/* If a tab with an id of 'notificationBtn' appears on the page, then the publisher can 
   run this function to see if the user is subscribed or not. The button must
   exist on the page prior to page load */
function updateBtn() {
  if (Notification.permission === 'denied') {
    pushButton.textContent = 'Push Messaging Blocked.';
    pushButton.disabled = true;
    sendUnsubscribeToBackEnd ();
    return;
  }

  if (isSubscribed) {
    pushButton.textContent = 'Click here to stop receiving notifications';
    /* this currently does nothing */
  } else {
    pushButton.textContent = 'Click here to receive notifications';
    /* this currently does nothing */
  }

  pushButton.disabled = false;
}

/* This fires when a user selects "Accept" in the subscripton dialog. Sends subscribe message to Push API */
function sendSubscribeToBackEnd(subscription) {
  /* Add subscription information returned from Notifications API to userSubscriptioninfo object */
  userSubscriptionInfo.eventType = 'Subscribe';
  userSubscriptionInfo.browserSubscription = subscription;
  console.log ("sendSubscribeToBackEnd userSubscriptionInfo: " + JSON.stringify(userSubscriptionInfo));

  return fetch('https://pushapi.content-ad.net/pushservice/users/subscriptions', {
    method: 'POST',
	crossDomain: true,
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(userSubscriptionInfo)
  })
  .then(function(response) {
    if (!response.ok) {
      console.log ('Error status code from server of ' + response.ok);
    }

    return response.json();
  })
  .then(function(responseData) {
    if (!(responseData.message)) {
      console.log ('Error response ' + responseData);
    }
    setCookie('pushId', userSubscriptionInfo.pushId, 3650);
    setCookie('subFlag', 'true', 3650);
    console.log ("responseData.message: " + responseData.message);
  });
}

/* Function is called when there is an error in sending a subscription to the Notifications API.
   Sends unsubscribe message to Push API */
function sendSubscribeDeclinedToBackEnd(subscription) {
  userSubscriptionInfo.eventType = 'Block';
  userSubscriptionInfo.browserSubscription = subscription;
  console.log ("sendSubscribeDeclinedToBackEnd userSubscriptionInfo: " + JSON.stringify(userSubscriptionInfo));

  return fetch('https://pushapi.content-ad.net/pushservice/users/subscriptions', {
    method: 'DELETE',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(userSubscriptionInfo)
  })
  .then(function(response) {
    if (!response.ok) {
      console.log ('Error status code from server of ' + response.ok);
    }
    setCookie('fpd',"true", 3650);
    return;
  });
}

/* Function is called when there is an error in sending a subscription to the Notifications API.
   Sends unsubscribe message to push api */
function sendUnsubscribeToBackEnd(subscription) {
  userSubscriptionInfo.eventType = 'Unsubscribe';
  userSubscriptionInfo.browserSubscription = subscription;
  console.log ("sendUnsubscribeToBackEnd userSubscriptionInfo: " + JSON.stringify(userSubscriptionInfo));

  var fpd = getCookie('fpd');
  if (! fpd) {
	  sendSubscribeDeclinedToBackEnd ();
	  return;
  }
  
  var subFlag = getCookie('subFlag');
  if ('false' == subFlag) {
	  return;
  }
  
  var pushId = getCookie('pushId');
  if ('' == pushId) {
	  return;
  }
  
  return fetch('https://pushapi.content-ad.net/pushservice/users/subscriptions', {
    method: 'PUT',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(userSubscriptionInfo)
  })
  .then(function(response) {
    if (!response.ok) {
      console.log ('Error status code from server of ' + response.ok);
    }

    return response.json();
  })
  .then(function(responseData) {
    if (!(responseData.message)) {
      console.log ('Error response ' + responseData);
    }
    setCookie('subFlag', 'false', 3650);
    console.log ("responseData.message: " + responseData.message);
  });
}

/* Function to notify the push api when a subscription impression has been served */
function sendImpressionToBackEnd(subscription) {

  var subImp = getCookie('subImp');
  if (subImp) {
    return;
  }

  var subImpInfo = {};

  subImpInfo.eventType = 'Prompt';
  subImpInfo.widgetId = scriptUrlParams ['wid'];
  subImpInfo.domainServed = window.location.hostname;
  subImpInfo.url = window.location.href.split(/[?#]/)[0];

  // subImpInfo.pushId = getCookie('pushId');

  console.log ("sendImpressionToBackEnd userSubscriptionInfo: " + JSON.stringify(subImpInfo));

  return fetch('https://pushapi.content-ad.net/pushservice/users/subscriptions', {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(subImpInfo)
  })
  .then(function(response) {
    if (!response.ok) {
      console.log ('Error status code from server of ' + response.ok);
    }
    setCookie('subImp', "true", 3650);

    return;
  });
}

/* Function fires when the user hits "Accept" and the Notifications API sucsessufly subscribes the user */
function subscribeUser() {
  const applicationServerKey = urlB64ToUint8Array(applicationServerPublicKey);
  swRegistration.pushManager.subscribe({
    userVisibleOnly: true,
    applicationServerKey: applicationServerKey
  })
  .then(function(subscription) {
    console.log('User is subscribed.');

    sendSubscribeToBackEnd (subscription);

    isSubscribed = true;
  })
  .catch(function(err) {
    console.log('Failed to subscribe the user: ', err);
    sendUnsubscribeToBackEnd(null);
  });
}

/* Orphan: sendUnsubscribeToBackEnd function is being called instead of this function */
function unsubscribeUser() {

  swRegistration.pushManager.getSubscription()
  .then(function(subscription) {
    if (subscription) {
      return subscription.unsubscribe();
    }
  })
  .catch(function(error) {
    console.log('Error unsubscribing', error);
  })
  .then(function() {
    sendUnsubscribeToBackEnd (subscription);

    console.log('User is unsubscribed.');
    isSubscribed = false;
  });
}

/* Request subscription data from Notificatons API. If the user has previously subscribed, 
   set isSubscribed to true. If the user is not subscribed, display the subscription dialog */
function initialize() {

  // Set the initial subscription value
  swRegistration.pushManager.getSubscription()
  .then(function(subscription) {
    isSubscribed = !(subscription === null);

    if (isSubscribed) {
      console.log('User IS subscribed.');
    } else {
      console.log('User is NOT subscribed. Asking the user to subscribe');
      subscribeUser();
      console.log('Asked the user to subscribe done');
    }
  });
}

/* If the service worker JS file has completed registration, 
   then initiate this push subscription script. If not, throw an error */
function registerServiceWork () {
	console.log('registerServiceWork entry  at ' + new Date());
	if ('serviceWorker' in navigator && 'PushManager' in window) {
	  console.log('Service Worker and Push is supported');

	  navigator.serviceWorker.register('/publisher_service_worker.js')
	  .then(function(swReg) {
	    console.log('Service Worker is registered', swReg);
	    swReg.update ();
	    swRegistration = swReg;
	    initialize();
	  })
	  .catch(function(error) {
	    console.error('Service Worker Error', error);
	  });
	} else {
	  console.warn('Push messaging is not supported');
	  // TO DO send msg to server
	    console.log('Push Not Supported', swReg);
	}	
}

/* Report to backend that a push subcription impresson has been served */
sendImpressionToBackEnd ();

var start = new Date().getTime();

/* When a delay was passed to this script as a URL param, then set a timeout 
   for the specified time, otherwise set a timeout of zero milliseconds */
setTimeout(registerServiceWork, delaySubPrompt);